package com.gct.dlmodel.service;

import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.gct.common.core.result.Result;
import com.gct.common.oss.OSSClient;
import com.gct.common.oss.OssClientUploadState;
import com.gct.dlmodel.entity.ModelResource;
import com.gct.dlmodel.feigncliet.DlViewClient;
import lombok.extern.slf4j.Slf4j;
import org.springframework.aop.interceptor.AsyncUncaughtExceptionHandler;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.scheduling.annotation.AsyncResult;
import org.springframework.stereotype.Service;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileOutputStream;
import java.io.OutputStreamWriter;
import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.Objects;
import java.util.concurrent.*;

/**
 * @author xusirui 2021/11/15
 */
@Service
public class OSSService {

    @Autowired
    OSSClient ossClient;

    @Autowired
    AsyncExec asyncExec;

    @Autowired
    DlViewClient dlViewClient;


    public Result uploadModel(String bucketName, String objectName, String localPath, Integer newVersion, String masterId, Integer partitionId) {
        File file = new File(localPath);
        if (!file.exists()) {
            return Result.ok("模型文件不存在,path:" + localPath);
        }

        Future<String> res = asyncExec.uploadFileAsync(bucketName, objectName + "." + newVersion, localPath, newVersion, masterId, partitionId);
        String s = null;
        try {
            s = res.get(200, TimeUnit.MILLISECONDS);
        } catch (Exception ignored) {

        }
        if (!Objects.isNull(s) && !s.isEmpty()) return Result.ok(s + ",file:" + file.getName());
        return Result.ok("上传任务已提交,file:" + file.getName());
    }

    @Service
    @Slf4j
    class AsyncExec {

        @Async
        public Future<String> uploadFileAsync(String bucketName, String objectName, String localPath,
                                              Integer newVersion, String masterId, Integer partitionId) {
            Result res = ossClient.uploadFile(bucketName, objectName, "false", localPath);
            System.out.println(bucketName + " " + objectName + " " + res);
            ModelResource modelResource = new ModelResource();
            modelResource.setUploadState(res.getStatus());
            modelResource.setUploadStateDesc(res.getMessage());
            modelResource.setMasterId(masterId);
            modelResource.setPartitionId(partitionId);
            modelResource.setUpdateTime(LocalDateTime.now());
            if (res.getStatus() == OssClientUploadState.COMPLETED.getId()) {
                modelResource.setVersionMark(newVersion.toString());
                modelResource.setFileSize(BigDecimal.valueOf(new File(localPath).length()));
                modelResource.setIsPublished(2);
            }
            CompletableFuture.runAsync(()->{
                try{
                    dlViewClient.updateModelResource(modelResource);
                }catch (Exception e){
                    log.error("call feign method updateModelResource failed,cause by {}",e.getMessage());
                }
            });
            return new AsyncResult<String>(res.getMessage());
        }
    }

}
